/*************************************************************************
 * All portions of code are copyright by their respective author/s.
 * Copyright (C) 2007      Bryan Christ <bryan.christ@hp.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------*/

#include <viper.h>

#include "vwm.h"

void vwm_keyring_set(gint keyring)
{
	VWM	*vwm;
	
	vwm=vwm_get_instance();
	vwm->active_keyring=keyring;
}

gint vwm_keyring_get(void)
{
	VWM 	*vwm;
	
	vwm=vwm_get_instance();
	return vwm->active_keyring;
}

gint vwm_hotkey_set(gint32 keystroke,VIPER_KEY_FUNC key_func,gint keyring)
{
	VWM				*vwm;
	VWM_HOTKEY		*vwm_hotkey;
	GSList			*node=NULL;
	
	vwm=vwm_get_instance();

	/*	look for an existing hotkey in the keyring list	*/
	if(g_slist_length(vwm->key_list)!=0)
	{
		node=vwm->key_list;
		while(node!=NULL)
		{
			vwm_hotkey=(VWM_HOTKEY*)node->data;
			if(vwm_hotkey->keystroke==keystroke &&
				vwm_hotkey->keyring==keyring) break;
			node=node->next;
		}
	}
	
	if(node!=NULL) vwm_hotkey->key_func=key_func;
	else
	{
		vwm_hotkey=(VWM_HOTKEY*)g_malloc0(sizeof(VWM_HOTKEY));
		vwm_hotkey->keystroke=keystroke;
		vwm_hotkey->key_func=key_func;
		vwm_hotkey->keyring=keyring;
		vwm->key_list=g_slist_prepend(vwm->key_list,(gpointer)vwm_hotkey);
	}
	
	return 0;
}

gint vwm_hotkey_swap(gint32 keystroke_old,gint32 keystroke_new,gint keyring)
{
	VWM			*vwm;
	VWM_HOTKEY	*vwm_hotkey;
	GSList		*node=NULL;
	
	vwm=vwm_get_instance();

	/*	look for an existing hotkey in the active keyring list	*/
	if(g_slist_length(vwm->key_list)==0) return -1;
	
	node=vwm->key_list;
	while(node!=NULL)
	{
		vwm_hotkey=(VWM_HOTKEY*)node->data;
		if(vwm_hotkey->keystroke==keystroke_old && 
			vwm_hotkey->keyring==keyring) break;
		node=node->next;
	}

	if(node==NULL) return -1;

	vwm_hotkey->keystroke=keystroke_new;
	return 0;
}

gint32 vwm_hotkey_run(gint32 keystroke,gpointer anything)
{
	VWM			*vwm;
	VWM_HOTKEY	*vwm_hotkey;
	GSList		*node=NULL;
	
	if(keystroke==-1) return -1;
	vwm=vwm_get_instance();
	if(vwm->active_keyring==-1) return keystroke;

	if(g_slist_length(vwm->key_list)!=0)
	{
		node=vwm->key_list;
		while(node!=NULL)
		{
			vwm_hotkey=(VWM_HOTKEY*)node->data;
			if(vwm_hotkey->keystroke==keystroke &&
				vwm_hotkey->keyring==vwm->active_keyring) break;
			node=node->next;
		}
	}

	if(node!=NULL)
	{
		if(vwm_hotkey->key_func!=NULL)
			vwm_hotkey->key_func(keystroke,anything);
		return -1;
	}
	
	return keystroke;
}
